/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2010 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2010.
 *   author Alberto Romeu aromeu@prodevelop.es
 */

package es.prodevelop.gvsig.mini.common;

import es.prodevelop.gvsig.mini.exceptions.BaseException;

/**
 * A singleton manager to register and retrieve compatible interfaces
 * implementations
 * 
 * @author aromeu
 * 
 */
public class CompatManager {

	private IContext mContext;
	private IEncryptor mEnc;
	private ILogHandler mLogHandler;

	private static CompatManager instance;

	/**
	 * A singleton
	 * 
	 * @return The CompatManager instance
	 */
	public static CompatManager getInstance() {
		if (instance == null) {
			instance = new CompatManager();
		}
		return instance;
	}

	/**
	 * Register an IContext instance. This method should be called once when
	 * initializing the app
	 * 
	 * @param aContext
	 */
	public void registerContext(IContext aContext) {
		this.mContext = aContext;
	}
	
	public void registerEncryptor(IEncryptor aEnc) {
		this.mEnc = aEnc;
	}

	/**
	 * Gets the previously registered IContext instance
	 * 
	 * @return The IContext instance
	 * @throws BaseException
	 *             If no instance whas previously registered
	 */
	public IContext getRegisteredContext() throws BaseException {
		if (this.mContext == null)
			throw new BaseException("No IContext registered");
		return this.mContext;
	}
	
	public IEncryptor getRegisteredEncIEncryptor() throws BaseException {
		if (this.mEnc == null)
			throw new BaseException("No Encryptor registered");
		return this.mEnc;
	}

	/**
	 * Register an ILogHandler instance. This method should be called once when
	 * initializing the app
	 * 
	 * @param aContext
	 */
	public void registerLogHandler(ILogHandler aLogHandler) {
		this.mLogHandler = aLogHandler;
	}

	/**
	 * Gets the previously registered ILogHandler instance
	 * 
	 * @return The ILogHandler instance
	 * @throws BaseException
	 *             If no instance was previously registered
	 */
	public ILogHandler getRegisteredLogHandler() throws BaseException {
		//FIXME
//		if (this.mLogHandler == null)
//			throw new BaseException("No ILogHandler registered");
		return this.mLogHandler;
	}
}
