/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2010 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2010.
 *   author Alberto Romeu aromeu@prodevelop.es
 */

package es.prodevelop.gvsig.mini.common.impl;

import java.util.Collection;
import java.util.Vector;
import java.util.logging.Level;

public abstract class CollectionQuickSort {

	public Object[] sort(Collection collection) {
		try {
			final int size = collection.size();

			Object[] temp = new Object[size];
			temp = collection.toArray();

			quicksort(temp);
			// collection = null;
			return temp;
		} catch (Exception e) {
			return null;
		}
	}

	/**
	 * Orders an array of points with the quicksort algorithm
	 * 
	 * @param a
	 *            The array of points to be ordered
	 * @param center
	 *            The point to compare with. The array will be sorted from
	 *            nearest to the center to furthest
	 */
	public void quicksort(final Object[] a) {
		quicksort(a, 0, a.length - 1);
	}

	// quicksort a[left] to a[right]
	private void quicksort(final Object[] a, final int left, final int right) {
		if (right <= left) {
			return;
		}
		int i = partition(a, left, right);
		quicksort(a, left, i - 1);
		quicksort(a, i + 1, right);
	}

	// partition a[left] to a[right], assumes left < right
	private int partition(final Object[] a, final int left, final int right) {
		int i = left;
		// int i = left - 1;
		// int j = right;
		int j = right + 1;
		Object v = a[left];
		while (true) {

			while (less(a[++i], v))
				if (i == right)
					break
					// find item on left to swap
					; // a[right] acts as sentinel

			while (less(v, a[--j])) // find
			// item
			// on
			// right
			// to
			// swap
			{
				if (j == left) {
					break; // don't go out-of-bounds

				}
			}
			if (i >= j) {
				break; // check if pointers cross

			}
			exch(a, i, j); // swap two elements into place

		}
		exch(a, left, j); // swap with partition element

		return j;
	}

	// is x < y ?
	public abstract boolean less(final Object x, final Object y);

	// exchange a[i] and a[j]
	private void exch(final Object[] a, final int i, final int j) {
		Object swap = a[i];

		a[i] = a[j];
		a[j] = swap;
	}

}
