/* gvSIG. Geographic Information System of the Valencian Government
*
* Copyright (C) 2007-2008 Infrastructures and Transports Department
* of the Valencian Government (CIT)
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
* MA  02110-1301, USA.
* 
*/

/*
* Prodevelop, S.L.
* Pza. Juan de Villarrasa, 14 - 5
* Valencia
* Spain
* Tel: +34 963510612
* e-mail: prode@prodevelop.es
* http://www.prodevelop.es
*
*/
package es.prodevelop.gvsig.mini.namefinder;

import es.prodevelop.gvsig.mini.utiles.Tags;
import es.prodevelop.gvsig.mini.utiles.Utilities;
import es.prodevelop.gvsig.mini.geom.Extent;
import es.prodevelop.gvsig.mini.geom.MultiPoint;
import es.prodevelop.gvsig.mini.geom.Point;

/**
 * A Multipoint of Named instances.
 * @author Alberto Romeu Carrasco (aromeu@prodevelop.es)
 */
public class NamedMultiPoint extends MultiPoint {

    /**
     * The constructor
     * @param points An array of named
     */
    public NamedMultiPoint(Named[] points) {
        super(points);
    }

    /**
     * Clears this instance
     */
    public void destroy() {
        final Named[] points = (Named[]) this.getPoints();
        final int size = points.length;

        for (int i = 0; i < size; i++) {
            points[i].destroy();
        }
        this.setPoints(null);
    }

    /**
     * Sets the projected coordinates of the Nameds in this instance
     * @param coords
     */
    public void setCoordinates(final Point[] coords) {
        try {
            final int size = coords.length;
            final Named[] list = (Named[]) this.getPoints();
            for (int i = 0; i < size; i++) {
                list[i].projectedCoordinates = coords[i];
            }
        } catch (final Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 
     * @return The projected coordinates of every Named in the NamedMultipoint
     */
    public Point[] getCoordinates() {
        try {
            final int size = this.getPoints().length;
            Point[] points = new Point[size];
            for (int i = 0; i < size; i++) {
                points[i] = ((Named) this.getPoint(i)).projectedCoordinates;
            }
            return points;
        } catch (final Exception e) {
            e.printStackTrace();
            return null;
        }

    }
}
